/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*******************************************************************************/

#include "system_config.h"
#include "system.h"
#include <xc.h>
#include <stdbool.h>
#include "sd_spi.h"

/** CONFIGURATION Bits **********************************************/
// CONFIG4
#pragma config DSWDTPS = DSWDTPS1F      // Deep Sleep Watchdog Timer Postscale Select bits (1:68719476736 (25.7 Days))
#pragma config DSWDTOSC = LPRC          // DSWDT Reference Clock Select (DSWDT uses LPRC as reference clock)
#pragma config DSBOREN = OFF            // Deep Sleep BOR Enable bit (DSBOR Disabled)
#pragma config DSWDTEN = OFF            // Deep Sleep Watchdog Timer Enable (DSWDT Disabled)
#pragma config DSSWEN = ON              // DSEN Bit Enable (Deep Sleep is controlled by the register bit DSEN)

// CONFIG3
#pragma config WPFP = WPFP127           // Write Protection Flash Page Segment Boundary (Page 127 (0x1FC00))
#pragma config VBTBOR = OFF             // VBAT BOR enable bit (VBAT BOR disabled)
#pragma config SOSCSEL = ON             // SOSC Selection bits (SOSC circuit selected)
#pragma config WDTWIN = PS25_0          // Watch Dog Timer Window Width (Watch Dog Timer Window Width is 25 percent)
#pragma config BOREN = ON               // Brown-out Reset Enable (Brown-out Reset Enable)
#pragma config WPDIS = WPDIS            // Segment Write Protection Disable (Disabled)
#pragma config WPCFG = WPCFGDIS         // Write Protect Configuration Page Select (Disabled)
#pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select (Write Protect from WPFP to the last page of memory)

// CONFIG2
#pragma config POSCMD = HS              // Primary Oscillator Select (HS Oscillator Enabled)
#pragma config BOREN1 = EN              // BOR Override bit (BOR Enabled [When BOREN=1])
#pragma config IOL1WAY = OFF            // IOLOCK One-Way Set Enable bit (The IOLOCK bit can be set and cleared using the unlock sequence)
#pragma config OSCIOFCN = ON            // OSCO Pin Configuration (OSCO/CLKO/RC15 functions as port I/O (RC15))
#pragma config FCKSM = CSDCMD           // Clock Switching and Fail-Safe Clock Monitor Configuration bits (Clock switching and Fail-Safe Clock Monitor are disabled)
#pragma config FNOSC = PRIPLL           // Initial Oscillator Select (Primary Oscillator with PLL module (XTPLL,HSPLL, ECPLL))
#pragma config ALTVREF = DLT_AV_DLT_CV  // Alternate VREF/CVREF Pins Selection bit (Voltage reference input, ADC =RA9/RA10 Comparator =RA9,RA10)
#pragma config IESO = ON                // Internal External Switchover (Enabled)

// CONFIG1
#pragma config WDTPS = PS32768          // Watchdog Timer Postscaler Select (1:32,768)
#pragma config FWPSA = PR128            // WDT Prescaler Ratio Select (1:128)
#pragma config FWDTEN = WDT_DIS         // Watchdog Timer Enable (WDT disabled in hardware; SWDTEN bit disabled)
#pragma config WINDIS = OFF             // Windowed WDT Disable (Standard Watchdog Timer)
#pragma config ICS = PGx2               // Emulator Pin Placement Select bits (Emulator functions are shared with PGEC2/PGED2)
#pragma config LPCFG = OFF              // Low power regulator control (Disabled)
#pragma config GWRP = OFF               // General Segment Write Protect (Disabled)
#pragma config GCP = OFF                // General Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (Disabled)


void SYSTEM_Initialize (void)
{

}


void MIKROBUS_A_SdSpiConfigurePins (void)
{
    _RP19R = 0x0007;   //RG8->SPI1:SDO1;
    _RP21R = 0x0008;   //RG6->SPI1:SCK1OUT;
    _SDI1R = 0x001A;   //RG7->SPI1:SDI1;
    
    //Settings for microSD click board using mikroBUS Interface A
    //CS = P14_CSA = RG9 
    //CD = P25_ANA_USBOC = RB10
    LATGbits.LATG9 = 0;
    TRISGbits.TRISG9 = 0;
    TRISBbits.TRISB0 = 1;
    
    ANSGbits.ANSG6 = 0;
    ANSGbits.ANSG7 = 0;
    ANSGbits.ANSG8 = 0;
    ANSBbits.ANSB0 = 0;
}

inline void MIKROBUS_A_SdSpiSetCs(uint8_t a)
{
    LATGbits.LATG9 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_A_SdSpiGetCd(void)
{
    return (!PORTBbits.RB0) ? true : false;

}

inline bool MIKROBUS_A_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusA =
{
    1,                                  // Use SPI module 1
    MIKROBUS_A_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_A_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_A_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_A_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};




void MIKROBUS_B_SdSpiConfigurePins (void)
{
    _SDI2R = 0x0017;   //RD2->SPI2:SDI2;
    _RP15R = 0x000A;   //RF8->SPI2:SDO2;
    _RP0R = 0x000B;   //RB0->SPI2:SCK2OUT;
    
    //Settings for microSD click board using mikroBUS Interface A
    //CS = P23_CSB = RB2 
    //CD = P24_ANB = RB1
    LATBbits.LATB2 = 0;
    TRISBbits.TRISB2 = 0;
    TRISBbits.TRISB1 = 1;
    
    ANSBbits.ANSB0 = 0;
    ANSBbits.ANSB1 = 0;
}

inline void MIKROBUS_B_SdSpiSetCs(uint8_t a)
{
    LATBbits.LATB2 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_B_SdSpiGetCd(void)
{
    return (!PORTBbits.RB1) ? true : false;

}

inline bool MIKROBUS_B_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusB =
{
    2,                                  // Use SPI module 1
    MIKROBUS_B_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_B_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_B_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_B_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};