/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*******************************************************************************/

#include "system_config.h"
#include "system.h"
#include <xc.h>
#include <stdbool.h>
#include "sd_spi.h"

/** CONFIGURATION Bits **********************************************/
// CONFIG3
#pragma config WPFP = WPFP511           // Write Protection Flash Page Segment Boundary (Highest Page (same as page 170))
#pragma config WPDIS = WPDIS            // Segment Write Protection Disable bit (Segmented code protection disabled)
#pragma config WPCFG = WPCFGDIS         // Configuration Word Code Page Protection Select bit (Last page(at the top of program memory) and Flash configuration words are not protected)
#pragma config WPEND = WPENDMEM         // Segment Write Protection End Page Select bit (Write Protect from WPFP to the last page of memory)

// CONFIG2
#pragma config POSCMOD = HS             // Primary Oscillator Select (HS oscillator mode selected)
#pragma config I2C2SEL = PRI            // I2C2 Pin Select bit (Use Default SCL2/SDA2 pins for I2C2)
#pragma config IOL1WAY = ON             // IOLOCK One-Way Set Enable bit (Write RP Registers Once)
#pragma config OSCIOFNC = ON            // Primary Oscillator Output Function (OSCO functions as port I/O (RC15))
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor (Both Clock Switching and Fail-safe Clock Monitor are disabled)
#pragma config FNOSC = PRIPLL           // Oscillator Select (Primary oscillator (XT, HS, EC) with PLL module (XTPLL,HSPLL, ECPLL))
#pragma config IESO = OFF               // Internal External Switch Over Mode (IESO mode (Two-speed start-up)disabled)

// CONFIG1
#pragma config WDTPS = PS32768          // Watchdog Timer Postscaler (1:32,768)
#pragma config FWPSA = PR128            // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Standard Watchdog Timer is enabled,(Windowed-mode is disabled))
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog Timer is disabled)
#pragma config ICS = PGx2               // Comm Channel Select (Emulator functions are shared with PGEC2/PGED2)
#pragma config GWRP = OFF               // General Code Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Code Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)



void SYSTEM_Initialize (void)
{
    AD1PCFGL = 0xFFFF;
}


void MIKROBUS_A_SdSpiConfigurePins (void)
{
    RPOR9bits.RP19R = 0x0007;   //RG8->SPI1:SDO1;
    RPOR10bits.RP21R = 0x0008;   //RG6->SPI1:SCK1OUT;
    RPINR20bits.SDI1R = 0x001A;   //RG7->SPI1:SDI1;
    
    //Settings for microSD click board using mikroBUS Interface A
    //CS = P14_CSA = RG9 
    //CD = P25_ANA_USBOC = RB10
    LATGbits.LATG9 = 0;
    TRISGbits.TRISG9 = 0;
    TRISBbits.TRISB0 = 1;
    
    AD1PCFGLbits.PCFG0 = 1;
}

inline void MIKROBUS_A_SdSpiSetCs(uint8_t a)
{
    LATGbits.LATG9 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_A_SdSpiGetCd(void)
{
    return (!PORTBbits.RB0) ? true : false;

}

inline bool MIKROBUS_A_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusA =
{
    1,                                  // Use SPI module 1
    MIKROBUS_A_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_A_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_A_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_A_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};




void MIKROBUS_B_SdSpiConfigurePins (void)
{
    RPINR22bits.SDI2R = 0x0017;   //RD2->SPI2:SDI2;
    RPOR7bits.RP15R = 0x000A;   //RF8->SPI2:SDO2;
    RPOR0bits.RP0R = 0x000B;   //RB0->SPI2:SCK2OUT;
    
    //Settings for microSD click board using mikroBUS Interface A
    //CS = P23_CSB = RB2 
    //CD = P24_ANB = RB1
    LATBbits.LATB2 = 0;
    TRISBbits.TRISB2 = 0;
    TRISBbits.TRISB1 = 1;
    
    AD1PCFGbits.PCFG1 = 1;
    AD1PCFGbits.PCFG2 = 1;
}

inline void MIKROBUS_B_SdSpiSetCs(uint8_t a)
{
    LATBbits.LATB2 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_B_SdSpiGetCd(void)
{
    return (!PORTBbits.RB1) ? true : false;

}

inline bool MIKROBUS_B_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusB =
{
    2,                                  // Use SPI module 1
    MIKROBUS_B_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_B_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_B_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_B_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};