/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*******************************************************************************/

#include "system_config.h"
#include "system.h"
#include <xc.h>
#include <stdbool.h>
#include "sd_spi.h"

/** CONFIGURATION Bits **********************************************/
// CONFIG2
#pragma config POSCMOD = HS             // Primary Oscillator Select (HS Oscillator mode selected)
#pragma config I2C1SEL = PRI            // I2C1 Pin Location Select (Use default SCL1/SDA1 pins)
#pragma config IOL1WAY = OFF            // IOLOCK Protection (IOLOCK may be changed via unlocking seq)
#pragma config OSCIOFNC = ON            // Primary Oscillator Output Function (OSC2/CLKO/RC15 functions as port I/O (RC15))
#pragma config FCKSM = CSDCMD           // Clock Switching and Monitor (Clock switching and Fail-Safe Clock Monitor are disabled)
#pragma config FNOSC = PRIPLL           // Oscillator Select (Primary Oscillator with PLL module (HSPLL, ECPLL))
#pragma config SOSCSEL = SOSC           // Sec Oscillator Select (Default Secondary Oscillator (SOSC))
#pragma config WUTSEL = LEG             // Wake-up timer Select (Legacy Wake-up Timer)
#pragma config IESO = OFF               // Internal External Switch Over Mode (IESO mode (Two-Speed Start-up) disabled)

// CONFIG1
#pragma config WDTPS = PS32768          // Watchdog Timer Postscaler (1:32,768)
#pragma config FWPSA = PR128            // WDT Prescaler (Prescaler ratio of 1:128)
#pragma config WINDIS = ON              // Watchdog Timer Window (Standard Watchdog Timer enabled,(Windowed-mode is disabled))
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog Timer is disabled)
#pragma config ICS = PGx2               // Comm Channel Select (Emulator EMUC2/EMUD2 pins are shared with PGC2/PGD2)
#pragma config GWRP = OFF               // General Code Segment Write Protect (Writes to program memory are allowed)
#pragma config GCP = OFF                // General Code Segment Code Protect (Code protection is disabled)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG port is disabled)

void SYSTEM_Initialize (void)
{
    AD1PCFG = 0xFFFF;
}


void MIKROBUS_A_SdSpiConfigurePins (void)
{
    _RP21R = 0x0007;   //RC5[RP21]->SPI1:SDO1;
    _RP24R = 0x0008;   //RC8[RP24]->SPI1:SCK1OUT;
    _SDI1R = 20;      //SPI1:SDI1<-RC4[RP20];
    
    //Settings for microSD click board using mikroBUS Interface A
    //CS = P14_CSA = RC2 
    //CD = P25_ANA_USBOC = RA0
    LATCbits.LATC2 = 0;
    TRISCbits.TRISC2 = 0;
    TRISAbits.TRISA0 = 1;
}

inline void MIKROBUS_A_SdSpiSetCs(uint8_t a)
{
    LATCbits.LATC2 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_A_SdSpiGetCd(void)
{
    return (!PORTAbits.RA0) ? true : false;
}

inline bool MIKROBUS_A_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusA =
{
    1,                                  // Use SPI module 1
    MIKROBUS_A_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_A_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_A_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_A_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};




void MIKROBUS_B_SdSpiConfigurePins (void)
{
    _SDI2R = 0x0017;   //SPI2:SDI2<-RC1[RP17];
    _RP16R = 0x000A;   //RC0[RP16]->SPI2:SDO2;
    _RP15R = 0x000B;   //RB0[RP15]->SPI2:SCK2OUT;
    
    //Settings for microSD click board using mikroBUS Interface B
    //CS = P23_CSB = RC6 
    //CD = P24_ANB = RB1
    LATCbits.LATC6 = 0;
    TRISCbits.TRISC6 = 0;
    TRISAbits.TRISA1 = 1;
}

inline void MIKROBUS_B_SdSpiSetCs(uint8_t a)
{
    LATCbits.LATC6 = a;     //mikroBUS click on interface B
}

inline bool MIKROBUS_B_SdSpiGetCd(void)
{
    return (!PORTAbits.RA1) ? true : false;
}

inline bool MIKROBUS_B_SdSpiGetWp(void)
{
    return false;
}

// The sdCardMediaParameters structure defines user-implemented functions needed by the SD-SPI fileio driver.
// The driver will call these when necessary.  For the SD-SPI driver, the user must provide
// parameters/functions to define which SPI module to use, Set/Clear the chip select pin,
// get the status of the card detect and write protect pins, and configure the CS, CD, and WP
// pins as inputs/outputs (as appropriate).
// For this demo, these functions are implemented in system.c, since the functionality will change
// depending on which demo board/microcontroller you're using.
// This structure must be maintained as long as the user wishes to access the specified drive.
FILEIO_SD_DRIVE_CONFIG mikroBusB =
{
    2,                                  // Use SPI module 1
    MIKROBUS_B_SdSpiSetCs,                    // User-specified function to set/clear the Chip Select pin.
    MIKROBUS_B_SdSpiGetCd,                    // User-specified function to get the status of the Card Detect pin.
    MIKROBUS_B_SdSpiGetWp,                    // User-specified function to get the status of the Write Protect pin.
    MIKROBUS_B_SdSpiConfigurePins             // User-specified function to configure the pins' TRIS bits.
};